local gr, pi, sin, cos, atan2	= love.graphics, math.pi, math.sin, math.cos, math.atan2
local sqrt, floor, max			= math.sqrt, math.floor, math.max
require 'spline'
function love.load()
	image		= gr.newImage( 'flower.jpg' )
	screen		= { w = image:getWidth(), h = image:getHeight() }
	love.window.setMode( screen.w, screen.h, { borderless = true } )
	fly = {
		image	= gr.newImage( 'fly.png' ),
		width	= 104,		-- spritesheet images not uniformally spaced
		height	= 75,
		sprites	= { {12,0}, {114,3}, {218,3}, {322,3}, {218,3}, {114,3} },
		scale	= 0.65,
		frame	= 1,
		tcur	= 0,		-- lapsed time
		tmax	= 0.03,		-- time limit (when frame should be changed)
		radius	= 200,
		center	= { screen.w/2+50, screen.h/2-20 },
		angle	= 0,
		update	= function( self, dt )
			self.tcur		= self.tcur + dt
			if self.tcur	> self.tmax then		-- change frame
				self.tcur	= 0						-- reset timer
				self.frame	= self.frame < #self.quads and self.frame+1 or 1
				fly.step = fly.step < #fly.pth and fly.step+1 or 2
				end
			self.angle 		= self.angle + dt
			self.x, self.y	= circularPath( self.angle, self.radius, self.center )
			end,
		draw	= function( f )
			gr.setColor( 255, 255, 255 )
			gr.draw( f.image, f.quads[f.frame], f.x, f.y, f.angle-pi, f.scale, f.scale, f.width/2, f.height/2 )
			end,
		cp	= { {200,150},{300,100},{420,100},{500,200},{500,300},{400,400},{300,400},
				{310,310},{400,250},{300,200},{200,300},{120,270},{150,200},{200,150}},
		step				= 2,
		showpath			= false,
		showcomtrolpoints	= false,
		showspline			= false,
		po					= 1		-- control point of interest (dragerable)
		}
	calcQuads( fly )
	fly.pth = smooth( fly.cp, 10 )
	end

function love.draw()
	gr.setColor( 255, 255, 255 )
	gr.draw( image, 0, 0 )
	fly:draw()
	gr.draw(fly.image,fly.quads[fly.frame],fly.pth[fly.step][1],fly.pth[fly.step][2],face(fly.pth[fly.step-1],fly.pth[fly.step],-pi/2),fly.scale,fly.scale,fly.width/2,fly.height/2)
	gr.setColor( 155, 0, 0 )
	if fly.showcontrolpoints then
		for i=1,#fly.cp do
			gr.circle( 'line', fly.cp[i][1], fly.cp[i][2], 5 )
			end
		gr.circle( 'fill', fly.cp[fly.po][1], fly.cp[fly.po][2], 5 )
		end
	if fly.showspline then
		for i = 1, #fly.pth do
			gr.circle( 'fill', fly.pth[i][1], fly.pth[i][2], 2 )
			end
		end
	if fly.showpath then
		for i = 2,#fly.pth do
			gr.line( fly.pth[i-1][1], fly.pth[i-1][2], fly.pth[i][1], fly.pth[i][2] )
			end
		gr.setColor( 0, 155, 0 )
		gr.circle( 'line', fly.center[1], fly.center[2], fly.radius )
		end
	end

function love.update( dt )
	fly:update( dt )
	mouse = { love.mouse.getPosition( ) }
	fly.po = closestPoint( fly.cp, mouse )
	if love.mouse.isDown( 'l' ) then
		fly.cp[fly.po] = mouse fly.pth = smooth( fly.cp, 10 )
		fly.showspline = true
		else
		fly.showspline = false
		end
	end

function love.keypressed( key )
	if key == "escape"	then love.event.quit() end
	if key == 'p'		then fly.showpath = not fly.showpath end
	if key == 'c'		then fly.showcontrolpoints = not fly.showcontrolpoints end
	end

function calcQuads( tab )		-- tab contains image, sprites(x,y[,w,h]), [width], [height] )
	tab.quads = {}
	for i = 1, #tab.sprites do
		tab.quads[ #tab.quads+1 ] = gr.newQuad(
			tab.sprites[i][1], tab.sprites[i][2],
			tab.sprites[i][3] or tab.width, tab.sprites[i][4] or tab.height,
			tab.image:getWidth(), tab.image:getHeight())
		end
	end

function circularPath( a, r, c )	-- angle, radius, center
	return r * cos( a ) + c[1], r * sin( a ) + c[2]
	end
